import pydicom 
import numpy as np
import cv2 

class ReferencedFrameOfReferenceSequence :

    def __init__(self): 
        pass


    def __create_ContourImageSequence(self, ReferencedSOPClassUID, list_all_SOPInstanceUID):
        ContourImageSequence = pydicom.sequence.Sequence()
        for SOPInstanceUID in list_all_SOPInstanceUID : 

            dataset = pydicom.dataset.Dataset()
            dataset.ReferencedSOPClassUID = ReferencedSOPClassUID
            dataset.ReferencedSOPInstanceUID = SOPInstanceUID
            ContourImageSequence.append(dataset)
        return ContourImageSequence 

    def __create_RTReferencedSeriesSequence(self, ReferencedSOPClassUID, list_all_SOPInstanceUID, SeriesInstanceUID):
        RTReferencedSeriesSequence = pydicom.sequence.Sequence()
         
        dataset = pydicom.dataset.Dataset()
        dataset.SeriesInstanceUID = SeriesInstanceUID
        dataset.ContourImageSequence = self.__create_ContourImageSequence(ReferencedSOPClassUID, list_all_SOPInstanceUID)
        RTReferencedSeriesSequence.append(dataset)
        return RTReferencedSeriesSequence

    def __create_RTReferencedStudySequence(self, ReferencedSOPClassUID, list_all_SOPInstanceUID, SeriesInstanceUID, StudyInstanceUID):
        RTReferencedStudySequence = pydicom.sequence.Sequence()
        dataset = pydicom.dataset.Dataset()
        dataset.ReferencedSOPInstanceUID = StudyInstanceUID
        dataset.ReferencedSOPClassUID = ReferencedSOPClassUID
        dataset.RTReferencedSeriesSequence = self.__create_RTReferencedSeriesSequence(ReferencedSOPClassUID, list_all_SOPInstanceUID, SeriesInstanceUID)
        RTReferencedStudySequence.append(dataset)
        return RTReferencedStudySequence

    def create_ReferencedFrameOfReferenceSequence(self, FrameOfReferenceUID, ReferencedSOPClassUID, list_all_SOPInstanceUID, SeriesInstanceUID, StudyInstanceUID): 
        ReferencedFrameOfReferenceSequence = pydicom.sequence.Sequence()
        dataset = pydicom.dataset.Dataset()
        dataset.FrameOfReferenceUID = FrameOfReferenceUID 
        dataset.RTReferencedStudySequence = self.__create_RTReferencedStudySequence(ReferencedSOPClassUID, list_all_SOPInstanceUID, SeriesInstanceUID, StudyInstanceUID)
        ReferencedFrameOfReferenceSequence.append(dataset)
        return ReferencedFrameOfReferenceSequence



import matplotlib
import numpy as np
from library_dicom.dicom_processor.model.csv_reader.Roi import Roi

"""Derivated Class for automatic Nifti ROI of PetCtViewer.org

Returns:
    [RoiNifti] -- Nifti ROI
"""
class RoiNifti(Roi):

    def __init__(self, roi_number, list_point, volume_dimension):
        super().__init__(1, 0, 0, roi_number, 0, list_point, volume_dimension)
        self.list_points = self.calculateMaskPoint()

    def calculateMaskPoint(self):
        pixel_array = self.list_point
        list_points = []
    
        for points in pixel_array :
            if len(points) != 0 : 

                list_points.append(points)

        return list_points


import pydicom 
import numpy as np
import cv2 
from random import randrange
from library_dicom.dicom_processor.model.reader.Instance_RTSS import Instance_RTSS
from library_dicom.export_segmentation.tools.rtss_writer_tools import *
from skimage.measure import label, find_contours
import matplotlib.pyplot as plt 
import SimpleITK as sitk 

class ROIContourSequence : 

    def __init__(self, mask_array, mask_img, number_of_roi):
        self.mask_array = mask_array 
        self.mask_img = mask_img
        self.number_of_roi = number_of_roi


    def __define_contour_img(self, number_roi):
        roi = np.zeros((self.mask_array.shape), dtype=np.uint8)
        z,x,y = np.where(self.mask_array == number_roi)
        roi[z,x,y] = 1
        roi_img = sitk.GetImageFromArray(roi)

        return roi_img 
        """
        depth = roi_img.GetDepth()
        contours = []
        for i in range(depth):
            im = roi_img[:,:,i]
            contour = sitk.BinaryContour(im, fullyConnected=True )
            contours.append(contour)
        vectorOfImages = sitk.VectorOfImage()
        for contour in contours:
            vectorOfImages.push_back(contour)
        img = sitk.JoinSeries(vectorOfImages)
        return img 
        """



    def get_spatial_coordonate(self, number_roi, list_all_SOPInstanceUID):
        img_roi = self.__define_contour_img(number_roi)
        depth = img_roi.GetDepth()
        results = []
        list_SOPInstance = []
        for z in range(depth):
            img_slice = img_roi[:,:,z]
            array_slice = sitk.GetArrayFromImage(img_slice)
            #liste = []
            contour = find_contours(array_slice, level = 0.0)
            if contour != []: 
                for i in range(len(contour)):
                    liste = []
                    l = contour[i].tolist()
                    for item in l:
                        spatial_coord = self.mask_img.TransformIndexToPhysicalPoint([int(item[1]), int(item[0]), int(z)])
                        liste.append(spatial_coord[0])
                        liste.append(spatial_coord[1])
                        liste.append(spatial_coord[2])

                    results.append(liste)
                    list_SOPInstance.append(list_all_SOPInstanceUID[z])

        return results, list_SOPInstance 

    """
    def __get_contour_ROI(self, number_roi):

        results = {}
        slice = []
        if self.mask.shape[-1] != 1 : 
            binary_mask = np.array(self.mask[:,:,:,number_roi - 1], dtype=np.uint8)
        else : 
            binary_mask = np.zeros((self.mask.shape[0], self.mask.shape[1], self.mask.shape[2]))
            x,y,z = np.where(self.mask[:,:,:,0] == number_roi)
            binary_mask[x,y,z] = 1
            binary_mask = np.array(binary_mask, dtype=np.uint8 )

        for s in range(self.mask.shape[2]):
            contours, _ = cv2.findContours(binary_mask[:,:, s], cv2.RETR_TREE,cv2.CHAIN_APPROX_NONE) 
            if (contours != []) : 
                results[s] = contours
                slice.append(s)

        return results, slice 


    def pixel_to_spatial(self, number_roi, image_position, pixel_spacing, list_all_SOPInstanceUID, liste_instances):
        liste_position = get_image_position_per_slice(liste_instances)


        list_contours = []
        list_SOPInstanceUID = []
        x0,y0,z0 = image_position
        dx,dy,dz = pixel_spacing
        dict_contours, list_slice = self.__get_contour_ROI(number_roi)

        number_contour = len(dict_contours)

        for i in range(number_contour): #plusieurs contours dans une même slice 
            number_of_contour_in_slice = len(dict_contours[list_slice[i]])
            for j in range(number_of_contour_in_slice):
                number_point_contour = len(dict_contours[list_slice[i]][j])

                liste = []

                for point in range(number_point_contour): #[x,y]
                    x = dict_contours[list_slice[i]][j][point][0][0]
                    liste.append(x0 + x*dx) #+dx/2
                    y = dict_contours[list_slice[i]][j][point][0][1]
                    liste.append( y0 + y*dy) #+dy/2
                    z = list_slice[i] 
                    z_spatial = find_corresponding_z_spatial(liste_position, z)
                    #liste.append(z0 + z*dz )
                    liste.append(z_spatial) 


                list_SOPInstanceUID.append(list_all_SOPInstanceUID[z])
                list_contours.append(liste)


        return list_contours, list_SOPInstanceUID
    """

    def __create_ContourImageSequence(self, ReferencedSOPClassUID, ReferencedSOPInstanceUID):
        ContourImageSequence = pydicom.sequence.Sequence()
        dataset = pydicom.dataset.Dataset()
        dataset.ReferencedSOPClassUID = ReferencedSOPClassUID
        dataset.ReferencedSOPInstanceUID = ReferencedSOPInstanceUID
        ContourImageSequence.append(dataset)
        return ContourImageSequence 

    def __create_ContourSequence(self, ReferencedSOPClassUID, list_ReferencedSOPInstanceUID, list_ContourData):
        ContourSequence = pydicom.sequence.Sequence()

        for ContourData,SOPInstanceUID in zip(list_ContourData,list_ReferencedSOPInstanceUID):
            dataset = pydicom.dataset.Dataset()
            dataset.ContourData = ContourData             
            dataset.ContourImageSequence = self.__create_ContourImageSequence(ReferencedSOPClassUID, SOPInstanceUID)             
            dataset.NumberOfContourPoints = len(ContourData)/3 
            dataset.ContourGeometricType = 'CLOSED_PLANAR'

            ContourSequence.append(dataset)
        return ContourSequence 


    @classmethod
    def get_random_colour(cls):
        max = 256
        return [randrange(max), randrange(max), randrange(max)]


    def create_ROIContourSequence(self, ReferencedSOPClassUID, list_all_SOPInstanceUID, liste_instances):
        ROIContourSequence = pydicom.sequence.Sequence()
        for number_roi in range(1, self.number_of_roi +1) : 
            #print("number_roi :", number_roi)
            dataset = pydicom.dataset.Dataset()
            dataset.ROIDisplayColor = self.get_random_colour()
            dataset.ReferencedROINumber = number_roi
            #list_contour_data , list_SOP_instance_uid = self.pixel_to_spatial(number_roi, image_position, pixel_spacing, list_all_SOPInstanceUID, liste_instances)
            list_contour_data, list_SOP_instance_uid = self.get_spatial_coordonate(number_roi, list_all_SOPInstanceUID )
            dataset.ContourSequence = self.__create_ContourSequence(ReferencedSOPClassUID, list_SOP_instance_uid, list_contour_data)
            ROIContourSequence.append(dataset)
        return ROIContourSequence 





import numpy as np
import SimpleITK as sitk 
from skimage.measure import label
#from radiomics.featureextractor import RadiomicsFeatureExtractor
import matplotlib.pyplot as plt
import scipy as sc
from sklearn import mixture



class Mask4D : 

    def __init__(self, mask_path): 
        self.mask_path = mask_path
        self.mask = self.read_mask_array()
        self.binary_mask = self.get_binary_mask()




    def read_mask_array(self):
        mask_img = sitk.ReadImage(self.mask_path)
        pixel_spacing = mask_img.GetSpacing()
        self.spacing = pixel_spacing #len  4 si mask  4D
        self.origin = mask_img.GetOrigin()
        self.direction = mask_img.GetDirection()
        self.size = mask_img.GetSize()
        return sitk.GetArrayFromImage(mask_img).transpose() #[x,y,z,channel]

    def get_threshold_mask(self, pet_array, threshold) : 
        mask_4D =  self.read_mask_array()
        number_of_roi = mask_4D.shape[3]
        for roi in range(number_of_roi) :
            #suv_values = []
            #suv_values.append(pet_array[np.where(mask_4D[:,:,:,roi] != 0 )])

            #maxi = np.max(suv_values)
            seuil = threshold
            #seuil = float(threshold)
            x,y,z = np.where(mask_4D[:,:,:,roi] != 0)
            for j in range(len(x)) :
                if pet_array[x[j],y[j],z[j]] <= seuil :
                    mask_4D[x[j],y[j],z[j],roi] = 0

        return mask_4D.astype(np.uint8)

    def get_binary_threshold_mask(self, pet_array, threshold) :
        threshold_mask =  self.get_threshold_mask(pet_array, threshold)
        shape_mask = threshold_mask.shape
        binary_threshold_mask = np.zeros((shape_mask[0], shape_mask[1], shape_mask[2]))
        sum_mask = np.ndarray.sum(threshold_mask, axis = -1)
        binary_threshold_mask[np.where(sum_mask != 0)] = 1
        return binary_threshold_mask.astype(np.uint8)



    def get_binary_mask(self) :
        shape_mask = self.mask.shape
        binary_mask = np.zeros((shape_mask[0], shape_mask[1], shape_mask[2]))
        sum_mask = np.ndarray.sum(self.mask, axis = -1)
        binary_mask[np.where(sum_mask != 0)] = 1
        return binary_mask.astype(np.uint8)


    def get_mask_spacing(self):
        return self.spacing[:3]

    def get_mask_size(self): 
        return self.size[:3]

    def get_mask_origin(self):
        return self.origin[:3]

    def get_mask_direction(self):
        return (self.direction[0], self.direction[1], self.direction[2], 
                            self.direction[4], self.direction[5], self.direction[6], 
                            self.direction[8],  self.direction[9], self.direction[10])



import numpy as np 
import SimpleITK as sitk 

#target_size = (128, 128, 256)
#target_spacing = (4.0, 4.0, 4.0)
#target_direction = (1,0,0,0,1,0,0,0,1)

class Fusion: 
    """A class to merge PET after resample 
    """


    def __init__(self, pet, ct, target_size, target_spacing, target_direction, mode='dict'):
        """[summary]

        Args:
            pet ([class object]): SeriesPT object or nifti image path or dict 
            ct([class object]): SeriesCT object or nifti image path or dict 
            target_size ([tuple]): [size (x,y,z)]
            target_spacing ([tuple]): [spacing (x,y,z)]
            target_direction ([tuple]): [direction ( x x x , y y y, z z z)]
            mode ([str]): 'serie' for serie object, 'img' if already PET and CT nifti path, 'dict' if dictionnary with img inside
        """
        self.pet_objet = pet
        self.ct_objet = ct
        self.target_size = target_size
        self.target_spacing = target_spacing
        self.target_direction = target_direction
        self.mode = mode

    def get_feature_pet_img(self):
        if self.mode == 'serie' : 
            instance_array = self.pet_objet.get_instances_ordered()

            original_pixel_spacing = instance_array[0].get_pixel_spacing()
            original_pixel_spacing = (float(original_pixel_spacing[0]), float(original_pixel_spacing[1]), self.serie_pet_objet.get_z_spacing())
            original_direction = instance_array[0].get_image_orientation()
            original_direction = (float(original_direction[0]), float(original_direction[1]), float(original_direction[2]),
                                    float(original_direction[3]), float(original_direction[4]), float(original_direction[5]),
                                    0.0,0.0,1.0)
            original_origin =instance_array[0].get_image_position()
            original_origin = (float(original_origin[0]), float(original_origin[1]), float(original_origin[2]))
            original_size = (self.pet_objet.get_size_matrix())
            return original_pixel_spacing, original_direction, original_origin, original_size

        elif self.mode == 'img' : 
            pet_img = sitk.ReadImage(self.pet_objet)
            original_pixel_spacing = pet_img.GetSpacing()
            original_direction = pet_img.GetDirection()
            original_origin = pet_img.GetOrigin()
            original_size = pet_img.GetSize()
            return original_pixel_spacing, original_direction, original_origin, original_size

        elif self.mode == 'dict' : 
            original_pixel_spacing = self.pet_objet.GetSpacing()
            original_direction = self.pet_objet.GetDirection()
            original_origin = self.pet_objet.GetOrigin()
            original_size = self.pet_objet.GetSize()
            return original_pixel_spacing, original_direction, original_origin, original_size

    def get_feature_ct_img(self):
        if self.mode == 'serie' : 
            instance_array = self.ct_objet.get_instances_ordered()

            original_pixel_spacing = instance_array[0].get_pixel_spacing()
            original_pixel_spacing = (float(original_pixel_spacing[0]), float(original_pixel_spacing[1]), self.serie_ct_objet.get_z_spacing())
            original_direction = instance_array[0].get_image_orientation()
            original_direction = (float(original_direction[0]), float(original_direction[1]), float(original_direction[2]),
                                    float(original_direction[3]), float(original_direction[4]), float(original_direction[5]),
                                    0.0,0.0,1.0)
            original_origin =instance_array[0].get_image_position()
            original_origin = (float(original_origin[0]), float(original_origin[1]), float(original_origin[2]))
            original_size = (self.ct_objet.get_size_matrix())
            return original_pixel_spacing, original_direction, original_origin, original_size

        elif self.mode == 'img' : 
            ct_img = sitk.ReadImage(self.ct_objet)
            original_pixel_spacing = ct_img.GetSpacing()
            original_direction = ct_img.GetDirection()
            original_origin = ct_img.GetOrigin()
            original_size = ct_img.GetSize()
            return original_pixel_spacing, original_direction, original_origin, original_size

        elif self.mode == 'dict' : 
            original_pixel_spacing = self.ct_objet.GetSpacing()
            original_direction = self.ct_objet.GetDirection()
            original_origin = self.ct_objet.GetOrigin()
            original_size = self.ct_objet.GetSize()
            return original_pixel_spacing, original_direction, original_origin, original_size

    def generate_pet_ct_img(self):
        if self.mode == 'serie' : 
            pet_spacing, pet_direction, pet_origin, _ = self.get_feature_pet_img()
            ct_spacing, ct_direction, ct_origin, _ = self.get_feature_ct_img()

            pet_array = self.pet_objet.get_numpy_array()
            ct_array = self.ct_objet.get_numpy_array()
            pet_img = sitk.GetImageFromArray(np.transpose(pet_array, (2,0,1)))
            pet_img.SetSpacing(pet_spacing)
            pet_img.SetOrigin(pet_origin)
            pet_img.SetDirection(pet_direction)


            ct_img = sitk.GetImageFromArray(np.transpose(ct_array, (2,0,1)))
            ct_img.SetSpacing(ct_spacing)
            ct_img.SetOrigin(ct_origin)
            ct_img.SetDirection(ct_direction)
            
            return pet_img, ct_img

        elif self.mode == 'img': 
            return sitk.ReadImage(self.pet_objet), sitk.ReadImage(self.ct_objet)

        elif self.mode == 'dict' : return self.pet_objet, self.ct_objet

    def calculate_new_origin(self, mode = 'head'):
        if mode == 'head' : return self.compute_new_origin_head2hips()

        elif mode == 'center' : return self.compute_new_origin_center()

    def compute_new_origin_head2hips(self):
        new_size = self.target_size
        new_spacing = self.target_spacing


        pet_spacing, _, pet_origin, pet_size = self.get_feature_pet_img()
        pet_origin = np.asarray(pet_origin)
        pet_size = np.asarray(pet_size)
        pet_spacing = np.asarray(pet_spacing)
        new_origin = (pet_origin[0] + 0.5 * pet_size[0] * pet_spacing[0] - 0.5 * new_size[0] * new_spacing[0],
                      pet_origin[1] + 0.5 * pet_size[1] * pet_spacing[1] - 0.5 * new_size[1] * new_spacing[1],
                      pet_origin[2] + 1.0 * pet_size[2] * pet_spacing[2] - 1.0 * new_size[2] * new_spacing[2])
        return new_origin

    def compute_new_origin_center(self):
        pet_spacing, _, pet_origin, pet_size = self.get_feature_pet_img()
        pet_origin = np.asarray(pet_origin)
        pet_size = np.asarray(pet_size)
        pet_spacing = np.asarray(pet_spacing)


        new_size = np.asarray(self.target_size)
        new_spacing = np.asarray(self.target_spacing)

        return tuple(pet_origin + 0.5 * (pet_size * pet_spacing - new_size * new_spacing))

        

    def resample(self, mode ='head'): 
        pet_img, ct_img = self.generate_pet_ct_img()
        new_origin = self.calculate_new_origin(mode=mode)
        
        #pet
        transformation = sitk.ResampleImageFilter()
        transformation.SetOutputDirection(self.target_direction)
        transformation.SetOutputOrigin(new_origin)
        transformation.SetOutputSpacing(self.target_spacing)
        transformation.SetSize(self.target_size)
        transformation.SetDefaultPixelValue(0.0)
        transformation.SetInterpolator(sitk.sitkBSpline)
        new_pet_img = transformation.Execute(pet_img) 

        #ct 
        transformation = sitk.ResampleImageFilter()
        transformation.SetOutputDirection(self.target_direction)
        transformation.SetOutputOrigin(new_origin)
        transformation.SetOutputSpacing(self.target_spacing)
        transformation.SetSize(self.target_size)
        transformation.SetDefaultPixelValue(-1000.0)
        transformation.SetInterpolator(sitk.sitkBSpline)
        new_ct_img = transformation.Execute(ct_img) 

        if self.mode == 'dict' : 
            return new_pet_img, new_ct_img


        pet_array, ct_array = sitk.GetArrayFromImage(new_pet_img), sitk.GetArrayFromImage(new_ct_img)
        concat = np.stack([pet_array, ct_array], axis = -1)
        return concat, new_origin #concat [z, y, x, number_channel]

    def save_nifti_fusion(self, pet_img, ct_img, filename, mode ='head'):
        """save merged PT/CT nifti after resample reshape 

        Args:
            pet_img ([type]): [description]
            ct_img ([type]): [description]
            filename ([type]): [description]
            mode (str, optional): [description]. Defaults to 'head'.

        Returns:
            [type]: 4D matrix, concatenate PT/CT 
        """
        concat, new_origin = self.resample(mode=mode) #[c, z, y, x]
        s = []
        for i in range(2): 
            img = sitk.GetImageFromArray(concat[:,:,:,i])
            img.SetSpacing(self.target_spacing)
            img.SetOrigin(new_origin)
            img.SetDirection(self.target_direction)
            s.append(img)
    
        #origin, spacing, direction from pet => 4D
        concat_img = sitk.JoinSeries(s)
        sitk.WriteImage(concat_img, filename)
        return concat_img







from numpy import random 
from library_dicom.dicom_processor.tools.folders import *



def generate_dict(number_of_roi, mode):
    #PAS PLUS DE 16 CARACTERES

    results = {}
    results["ContentCreatorName"] = "dicom_to_cnn"
    random_number = random.randint(0,1e3)
    
    name = str(input('Enter filename for json file (filename or nothing):'))
    if not name : results["ClinicalTrialSeriesID"] = "json_serie"+str(random_number)
    else : 
        if len(name) > 16 : results["ClinicalTrialSeriesID"] = name[0:16]
        else : results["ClinicalTrialSeriesID"] = name #name of json

    results["ClinicalTrialTimePointID"] = "1"

    description = str(input('Enter serie description (serie description or nothing):'))
    if not description : results["SeriesDescription"] =''
    else : 
        if len(description) > 16 : results["SeriesDescription"] = description[0:16]
        else : results["SeriesDescription"] = description
    
    results["SeriesNumber"] = str(random_number)
    results["InstanceNumber"] = str(number_of_roi)

    body_part = str(input('Enter body part examined (body part or nothing) : '))
    if not body_part : results["BodyPartExamined"] = "all body"
    else : 
        if len(body_part) > 16 : results["BodyPartExamined"] = body_part[0:16]
        else : results["BodyPartExamined"] = body_part

    results["segmentAttributes"] = []
    subliste = []
    for i in range(number_of_roi):
        subdict = {}
        subdict["labelID"]= i+1
        segment_description = str(input('Enter ROI n°{} description/name (segment description or nothing) : '.format(i+1)))
        if not segment_description : subdict["SegmentDescription"] = str("ROI {}".format(i+1))
        else : 
            if len(segment_description) > 16 : 
                subdict["SegmentDescription"] = segment_description[0:16]
            else : subdict["SegmentDescription"] = segment_description

        subdict["SegmentAlgorithmType"] = "SEMIAUTOMATIC"
        subdict["SegmentAlgorithmName"] =  "DeepOncology"
        if mode == 'rtstruct' : 
            RTROIInterpretedType = str(input("Enter RTROIInterpredType for ROI n°{} (predefined type or nothing) : ".format(i+1)))
            if not RTROIInterpretedType : subdict['RTROIInterpretedType'] = ''
            else : subdict['RTROIInterpretedType'] = RTROIInterpretedType
        


        #subdict["SegmentedPropertyCategoryCodeSequence"] 
        subsubdict = {}
        subsubdict["CodeValue"] = "49755003"
        subsubdict["CodingSchemeDesignator"] = "SCT"
        subsubdict["CodeMeaning"] = "Morphologically Altered Structure"
        subdict["SegmentedPropertyCategoryCodeSequence"] = subsubdict

        #subdict["SegmentedPropertyTypeCodeSequence"] = {}
        subsubdict = {}
        subsubdict["CodeValue"] = "108369006"
        subsubdict["CodingSchemeDesignator"] = "SCT"
        subsubdict["CodeMeaning"] = "Neoplasm"
        subdict["SegmentedPropertyTypeCodeSequence"] = subsubdict
        subliste.append(subdict)

    results["segmentAttributes"].append(subliste)

    results["ContentLabel"] =  "SEGMENTATION"
    results["ContentDescription"] = "Image segmentation"
    results["ClinicalTrialCoordinatingCenterName"] = "dicom_to_cnn"

    return results

def save_dict_as_json(results, directory):
    filename = results["ClinicalTrialSeriesID"]
    write_json_file(directory, filename, results)
    return os.path.join(directory, filename+'.json')



import os
import json

def get_series_path(path):
    """Go through all the folder to find every series path

    Arguments:
        path {[string]} -- [Absolute path where the repertory is located]

    Returns:
        [list] -- [Path's list of every series]
    """

    seriesPath = []
    for (path, dirs, files) in os.walk(path): 
        if not (dirs) :
            seriesPath.append(path)
    return seriesPath

"""
ATTENTION A TOUT EFFACE
def remove_empty_folders(path):

    empty_folder = []
    for (path, dirs, files) in os.walk(path): 
        if (len(files) ==0) :
            empty_folder.append(path)
    return empty_folder
"""

def write_json_file(path, file_name, content, extension = 'json'):
    wrinting_file = open(os.path.join(path, file_name+'.'+extension), 'w')
    wrinting_file.write(json.dumps(content))
    wrinting_file.close()

def remove_bi_file(path):
    file_names = os.listdir(path)
    if('graphic.brownFat.gr2' in file_names):
        os.remove( os.path.join(path,"graphic.brownFat.gr2") )

def remove_index_ini(path):
    file_names = os.listdir(path)
    if ('index.ini' in file_names):
        os.remove(os.path.join(path, 'index.ini'))



from library_dicom.dicom_processor.model.csv_reader.RoiPolygon import RoiPolygon
from library_dicom.dicom_processor.model.csv_reader.RoiElipse import RoiElipse
from library_dicom.dicom_processor.model.csv_reader.RoiNifti import RoiNifti

"""Create ROI object from a CSV Roi Details
    ROI Object is able to build the Nifti Mask according to CSV informations

Returns:
    [RoiFactory] -- Factory for ROI instanciation according to csv details
"""
class RoiFactory():

    def __init__(self, details, volume_dimension, roi_number):
       """[summary]

       Arguments:
           details {dict} -- Parsed ROI informations
           volume_dimension {(x,y,z)} -- Matrice dimension for Mask (should match DICOM input size)
           roi_number {int} -- Integer number that will have to be written in the mask
       """
       self.details = details
       self.volume_dimension = volume_dimension
       self.roi_number = roi_number
    
    def read_roi(self):
        """Instanciate the Correct ROI type according to data

        Returns:
            [Roi] -- Roi object
        """
        first_slice = self.details['first_slice']
        last_slice = self.details['last_slice']
        point_list  = self.details['point_list']
        roi_number = self.roi_number
        type_number = self.details['type_number']
        volume_dimension = self.volume_dimension

        if (self.details['type_number'] == 1):
            return RoiPolygon( 1, first_slice, last_slice, roi_number, type_number, point_list, volume_dimension )

        elif (self.details['type_number'] == 11):
            return RoiElipse( 1, first_slice, last_slice, roi_number, type_number, point_list, volume_dimension )

        elif (self.details['type_number'] == 2):
            return RoiPolygon( 2, first_slice, last_slice, roi_number, type_number, point_list, volume_dimension )

        elif (self.details['type_number'] == 12):
            return RoiElipse( 2, first_slice, last_slice, roi_number, type_number, point_list, volume_dimension )

        elif (self.details['type_number'] == 3):
            return RoiPolygon( 3, first_slice, last_slice, roi_number, type_number, point_list, volume_dimension )

        elif (self.details['type_number'] == 13):
            return RoiElipse( 3, first_slice, last_slice, roi_number, type_number, point_list, volume_dimension )
        
        else:
            return RoiNifti( roi_number, point_list, volume_dimension )


    


from library_dicom.dicom_processor.model.Series import Series
from library_dicom.dicom_processor.model.reader.Instance import Instance
import pydicom
import os
import cv2 as cv2
import numpy as np 
import sys 



class Instance_RTSS(Instance):
    """Class to read dicom RT file and build mask 
    """

    def __init__(self, path_rtss):
        super().__init__(path_rtss, load_image=True)


    def get_image_nparray(self):
        sys.exit('Cannot get image numpy array from RTSTRUCT FILE')


    def get_list_all_SOP_Instance_UID_RTSS(self):
        number_item = len(self.dicomData[0x30060010][0].RTReferencedStudySequence[0].RTReferencedSeriesSequence[0].ContourImageSequence)
        liste = []
        for i in range(number_item):
            liste.append(str(self.dicomData[0x30060010][0].RTReferencedStudySequence[0].RTReferencedSeriesSequence[0].ContourImageSequence[i].ReferencedSOPInstanceUID))
        
        return liste



    def get_number_of_roi(self):
        return len(self.dicomData.StructureSetROISequence)


    #info from ReferencedFrameOfReferenceSequence
    def get_number_of_referenced_series(self):
        """return number of series in ReferencedFrameOfReferencedSequence

        """
        return len(self.dicomData.ReferencedFrameOfReferenceSequence)


    def get_frame_of_reference_uid(self):
        #Doit etre le meme que FrameOfReferenceUID de la serie CT/PT
        number_serie = len(self.dicomData.ReferencedFrameOfReferenceSequence)
        liste = []
        for i in range(number_serie):
            liste.append(self.dicomData[0x30060010][i].FrameOfReferenceUID)

        return liste #les mêmes pour les 2 series 



    def is_frame_of_reference_same(self):
        """check if frame of reference is the same for the series in ReferencedOfReferencedSequence

        """
        frame_of_reference_UID = self.get_frame_of_reference_uid()
        for i in range(len(frame_of_reference_UID)):
            if frame_of_reference_UID[0] != frame_of_reference_UID[i] :
                return False 
        return True 


    def get_referenced_series_instance_uid(self):
        #doit être le même que SeriesInstanceUID de la CT/PT
        """get series instance UID from the CT/PT serie 


        """
        number_serie = len(self.dicomData.ReferencedFrameOfReferenceSequence)
        liste = []
        for i in range(number_serie):
            liste.append(self.dicomData[0x30060010][i].RTReferencedStudySequence[0].RTReferencedSeriesSequence[0].SeriesInstanceUID)
        return liste


    def get_referenced_study_SOP_instance_uid(self):
        #Doit etre le meme que la StudyInstanceUID de la CT/PT + meme que StudyInstanceUID du RTSS
        number_serie = len(self.dicomData.ReferencedFrameOfReferenceSequence)
        liste = []
        for i in range(number_serie):
            liste.append(self.dicomData[0x30060010][i].RTReferencedStudySequence[0].ReferencedSOPInstanceUID)
        return liste


    def get_referenced_SOP_class_UID(self):
        #Doit etre le meme que SOPClassUID de la serie CT/PT
        number_serie = len(self.dicomData.ReferencedFrameOfReferenceSequence)
        liste = []
        for i in range(number_serie):
            liste.append(self.dicomData[0x30060010][i].RTReferencedStudySequence[0].RTReferencedSeriesSequence[0].ContourImageSequence[0].ReferencedSOPClassUID)
        return liste 

 
    #info from StructureSetROISequence
    def get_ROI_name(self, number_roi):
        return self.dicomData[0x30060020][number_roi - 1].ROIName 

    def get_ROI_volume(self, number_roi):
        return self.dicomData[0x30060020][number_roi - 1].ROIVolume 

    def get_ROI_generation_algorithm(self, number_roi):
        return self.dicomData[0x30060020][number_roi - 1].ROIGenerationAlgorithm 

    #info from ROIContourSequence

    def get_roi_display_color(self, number_roi):
        return self.dicomData[0x30060039][number_roi - 1].ROIDisplayColor 


    def get_list_referenced_SOP_Instance_UID(self, roi_number): #la ou il y a les contours
        number_item = len(self.dicomData[0x30060039][roi_number - 1].ContourSequence)
        liste = []
        for i in range(number_item):
            liste.append(str(self.dicomData[0x30060039][roi_number - 1].ContourSequence[i].ContourImageSequence[0].ReferencedSOPInstanceUID))
        return liste 

    
    def is_referenced_SOP_Instance_UID_in_all_SOP_Instance(self):
        """check if slices in which there is a ROI is in the list of all slices of the serie

        """
        all_sop = str(self.get_list_all_SOP_Instance_UID_RTSS)
        referenced_sop = str(self.get_list_referenced_SOP_Instance_UID)
        for uid in referenced_sop : 
            if uid not in all_sop : 
                raise Exception("SOP Instance UID not in the serie")
        return True 


    def get_number_of_contour_points(self, roi_number):
        number_item = len(self.dicomData[0x30060039][roi_number - 1].ContourSequence)
        liste = []
        for i in range(number_item):
            liste.append(self.dicomData[0x30060039][roi_number - 1].ContourSequence[i].NumberOfContourPoints)
        return liste

    def get_list_contour_geometric_type(self, roi_number):
        number_item = len(self.dicomData[0x30060039][roi_number - 1].ContourSequence)
        liste = []
        for i in range(number_item):
            liste.append(self.dicomData[0x30060039][roi_number - 1].ContourSequence[i].ContourGeometricType)
        return liste

    def is_closed_planar(self, roi_number):
        geometric_type = self.get_list_contour_geometric_type(roi_number)
        for type in geometric_type :
            if type != "CLOSED_PLANAR" : 
                return False
        return True 

    def get_contour_data(self, roi_number):
        number_item = len(self.dicomData[0x30060039][roi_number - 1].ContourSequence)
        liste = []
        for i in range(number_item):
            liste.append(self.dicomData[0x30060039][roi_number - 1].ContourSequence[i].ContourData)
        return liste #liste d'array 


    def get_ROIName_by_number(self, number):
        for i in range(len(self.dicomData.StructureSetROISequence)):
            if int(self.dicomData.StructureSetROISequence[i].ROINumber) == number : 
                return self.dicomData.StructureSetROISequence[i].ROIName 


from enum import Enum

class TagsPatient(Enum):
    """Patient related dicom tags

    Arguments:
        Enum {hex} -- [Tag address]
    """

    PatientName = 0x00100010
    PatientID = 0x00100020
    PatientBirthDate = 0x00100030
    PatientSex = 0x00100040

class TagsStudy(Enum):
    """Study related tags

    Arguments:
        Enum {hex} -- [Tag address]
    """

    AccessionNumber = 0x00080050
    InstitutionName = 0x0080080
    StudyDate = 0x00080020
    StudyTime = 0x00080030
    StudyDescription = 0x00081030
    StudyID = 0x00200010
    StudyInstanceUID = 0x0020000D
    PatientWeight = 0x00101030
    PatientHeight = 0x00101020

class TagsSeries(Enum):
    """Series related dicom tags

    Arguments:
        Enum {hex} -- [Tag address]
    """
    Manufacturer = 0x00080070
    Modality = 0x00080060
    SeriesDate = 0x00080021
    SeriesTime = 0x00080031
    SeriesDescription = 0x0008103E
    SeriesInstanceUID = 0x0020000E
    SeriesNumber = 0x00200011
    AcquisitionDate = 0x00080022
    AcquisitionTime = 0x00080032
    NumberOfSlices = 0x00540081
    ImageInAcquisition = 0x00201002
    Units = 0x00541001
    DecayCorrection = 0x00541102

class PhilipsPrivateTags(Enum):
    PhilipsSUVFactor = 0x70531000
    PhilipsBqMlFactor = 0x70531009

class TagPTCorrection(Enum):
    CorrectedImage = 0x00280051

class ImageType(Enum):
    ImageType = 0x00080008

class PixelSpacing(Enum):
    PixelSpacing = 0x00280030


class TagsInstance(Enum):
    """Instance related tags

    Arguments:
        Enum {hex} -- [tag address]
    """

    ImagePosition = 0x00200032
    ImageOrientation = 0x00200037
    RescaleSlope = 0x00281053
    RescaleIntercept = 0x00281052
    SOPInstanceUID = 0x00080018

class TagsRadioPharmaceuticals(Enum):
    """Tags related to Radiopharmaceuticals injection

    Arguments:
        Enum {hex} -- [tag address]
    """

    RadionuclideHalfLife = 0x00181075
    TotalDose = 0x00181074
    RadiopharmaceuticalStartDateTime = 0x00181078
    RadiopharmaceuticalStartTime = 0x00181072


import pydicom 
import numpy as np
from library_dicom.export_segmentation.tools.rtss_writer_tools import *

class StructureSetROISequence : 

    def __init__(self, mask_array, results, number_of_roi):
        self.mask_array = mask_array
        self.number_of_roi = number_of_roi
        self.results = results


    def create_StructureSetROISequence(self, pixel_spacing, ReferencedFrameOfReferenceUID) :
        StructureSetROISequence = pydicom.sequence.Sequence()

        for number_roi in range(1 , self.number_of_roi +1):
            dataset = pydicom.dataset.Dataset()
            dataset.ROINumber = number_roi
            dataset.ReferencedFrameOfReferenceUID = ReferencedFrameOfReferenceUID             
            dataset.ROIName = self.results['segmentAttributes'][0][number_roi-1]["SegmentDescription"]
            dataset.ROIVolume = str(self.get_roi_volume(number_roi, pixel_spacing))
            dataset.ROIGenerationAlgorithm = self.results['segmentAttributes'][0][number_roi-1]["SegmentAlgorithmType"]
            StructureSetROISequence.append(dataset)
        return StructureSetROISequence 
        
     
    def get_roi_volume(self, number_roi, pixel_spacing):
        number_pixel = 0
        if len(self.mask_array.shape) == 3 :
            _, x, _ = np.where(self.mask_array == number_roi)
        elif len(self.mask_array.shape) == 4 : 
            raise Exception ('4D array. Need a 3D array with [z,x,y]')
        number_pixel = len(x) #same as len(y) or len(z)
        volume_pixel = pixel_spacing[0] * pixel_spacing[1] * abs(pixel_spacing[2])
        volume_pixel = volume_pixel * 10**(-3) #mm3 to ml 
        roi_volume = number_pixel * volume_pixel 

        return np.round(roi_volume , 5)




import numpy as np
import SimpleITK as sitk 
from skimage.measure import label
#from radiomics.featureextractor import RadiomicsFeatureExtractor
import matplotlib.pyplot as plt
import scipy as sc
from sklearn import mixture




class Mask3D : 

    def __init__(self, mask_path): 
        self.mask_path = mask_path
        self.mask = self.read_mask_array()

    
    def read_mask_array(self):
        mask_img = sitk.ReadImage(self.mask_path)
        pixel_spacing = mask_img.GetSpacing()
        self.spacing = pixel_spacing #len 3 si mask 3D 
        self.origin = mask_img.GetOrigin()
        self.direction = mask_img.GetDirection()
        self.size = mask_img.GetSize()
        return sitk.GetArrayFromImage(mask_img).transpose() #[x,y,z]


    def get_binary_threshold_mask(self, threshold): #0.5
        size = self.mask.shape
        threshold_mask = np.zeros(size)
        x,y,z = np.where(self.mask > threshold)
        for j in range(len(x)):
            threshold_mask[x[j], y[j], z[j]] = 1

        return threshold_mask.astype(np.uint8)
        

    def get_mask_spacing(self):
        return self.spacing

    def get_mask_size(self) :
        return self.size

    def get_mask_origin(self):
        return self.origin

    def get_mask_direction(self):
        return self.direction


import numpy as np

class Roi():

    def __init__(self, axis, first_slice, last_slice, roi_number, type_number, list_point, volume_dimension):
        self.axis = axis
        self.first_slice = first_slice
        self.last_slice = last_slice
        self.roi_number = roi_number
        self.type_number = type_number
        self.list_point = list_point
        self.list_point_np = np.asarray(self.list_point)
        self.x = volume_dimension[0]
        self.y = volume_dimension[1]
        self.z = volume_dimension[2]
        
    
    def __get_min_max_of_roi(self):
        """Compute extrema of ROI in which we will loop to find included voxel

        Arguments:
            point_list {np array} -- numpy point list

        Returns:
            [minx, miny, maxx, maxy] -- X/Y extremas
        """
        points_array = self.list_point_np

        all_x = points_array[:][:,0]
        all_y = points_array[:][:,1]

        if (self.type_number == 1 or self.type_number == 2 or self.type_number == 3) : #si polygone


            xmin = min(all_x)
            xmax = max(all_x)
            ymin = min(all_y)
            ymax = max(all_y)
        
            return xmin , xmax , ymin , ymax 
        else : #ellipse
            height =  abs(all_x[0] - all_x[1])
            width = abs(all_y[0] - all_y[2])
            xmin = all_x[0] - height
            xmax = all_x[0] + height
            ymin = all_y[0] -  width
            ymax = all_y[0] + width 
            return xmin , xmax , ymin, ymax


    def mask_roi_in_slice(self, patch): #patch = ellipse ou polygone 
        #get Roi limits in wich we will loop
        points = []
        xmin, xmax, ymin, ymax  = self.__get_min_max_of_roi()
        for x in range(xmin, xmax + 1): 
            for y in range(ymin, ymax + 1) : 
                if patch.contains_point([x,y], radius = -1e-9) : 
                    points.append([x,y])

        return points
    
    def get_empty_np_array(self):
        """Return numpy array to fill given the current dimension and axis

        Returns:
            numpy array -- zero filled numpy array
        """

        #change les coordonnées dans mask_builder pour passer en axial directement donc pas besoin 
        #ici de différencier axial, saggital et coronal 
        #if (self.axis == 1):
        return (np.zeros((self.x, self.y, self.z)))
            
       # elif (self.axis == 2):
            #return (np.zeros((self.z, self.y, self.x)))
            

        #elif (self.axis == 3):
            #return (np.zeros((self.z, self.y, self.x)))

    def coronal_to_axial(self, np_array_3D):
        return np.transpose(np_array_3D, (2,1,0))
        #return np.transpose(np_array_3D, (0,2,1))
         #coronnal x y z -> axial x z y
        #SK A VERIFRO

    def sagittal_to_axial(self, np_array_3D):
        return np.transpose(np_array_3D, (0,2,1))
         #sagittal x y z - > axial y z x   
        #SK A VERIF

    def get_mask(self, list_points): #list_points = [[x,y,z], [x,y,z], ...]
        np_array_3D = self.get_empty_np_array()
        for point in list_points:
            np_array_3D[point[1], point[0] , point[2]] = 1
            #x et y inversé dans matplotlib 

        #if (self.axis == 2) : 
            #np_array_3D = self.coronal_to_axial(np_array_3D)
            #print("shape après coronal to axial", np_array_3D.shape)
            #return np.transpose(np_array_3D, (1,0,2)).astype(np.uint8)
            #return np_array_3D.astype(np.uint8)
        #elif (self.axis == 3) :
            #np_array_3D = self.sagittal_to_axial(np_array_3D)
            #return np_array_3D.astype(np.uint8)

        return np_array_3D.astype(np.uint8)



    

        
    









import os
import imageio
from os.path import basename,splitext
import scipy 
import numpy as np
import matplotlib.pyplot as plt

from scipy import ndimage
from PIL import Image
from fpdf import FPDF


def create_gif(filenames, duration, name, directory):
    """From a list of images, create gif

    Args:
        filenames ([list]): [list of all images' path]
        duration ([float]): [time of each image]
        name ([str]): [gif name]
        directory ([str]): [gif directory]


    """
    images = []
    for filename in filenames:
        images.append(imageio.imread(filename))
        
    #name = splitext(basename(filenames[0][:-2]))[0]
    output_file = directory+'/' + name +'.gif'
    imageio.mimsave(output_file, images, duration=duration)
    
    return None

def mip_imshow(numpy_array, angle, type, cmap, vmin, vmax):
    """function to visualize MIP of 3D array 

    Args:
        numpy_array ([type]): [description]
        angle ([type]): [description]
        cmap ([type]): [description]
        type ([type]): [description]
        vmin ([type]): [description]
        vmax ([type]): [description]
    """
    numpy_array = np.transpose(np.flip(numpy_array, axis = 2), (2,1,0)) #coronal

    vol_angle = scipy.ndimage.interpolation.rotate(numpy_array , angle , reshape=False, axes = (1,2))
    MIP = np.amax(vol_angle,axis=2)

    f = plt.figure(figsize=(10,10))
    axes = plt.gca()
    axes.set_axis_off()
    if type == 'mask' : 
        plt.imshow(MIP, cmap = cmap)
        plt.show()
    else : 
        plt.imshow(MIP, cmap = cmap, vmin = vmin, vmax = vmax) #vmax = 3.0
        plt.show()

def mip_projection(numpy_array, angle, path_image, study_uid, type, cmap, vmin, vmax):
    """function to save MIP of 3D array 

    Args:
        numpy_array ([ndarray]): [PET or MASK 3D array]
        angle ([int]): [description]
        path_image ([str]): [description]
        study_uid ([str]): [description]
        type ([str]): ['pet' or 'mask']
        cmap ([str]): [description]
        vmin ([int]): [description]
        vmax ([int]): [description]

    Returns:
        save as png the MIP
    """
    numpy_array = np.transpose(np.flip(numpy_array, axis = 2), (2,1,0)) #coronal

    vol_angle = scipy.ndimage.interpolation.rotate(numpy_array , angle , reshape=False, axes = (1,2))
    MIP = np.amax(vol_angle,axis=2)

    f = plt.figure(figsize=(10,10))
    axes = plt.gca()
    axes.set_axis_off()
    if type == 'pet' :
        plt.imshow(MIP, cmap = cmap, vmin = vmin, vmax = vmax)
        filename = study_uid+'_mip_'+type+"_"+str(int(angle))+".png"

    else : 
        plt.imshow(MIP, cmap = cmap)
        filename = study_uid+'_mip_'+type+"_"+str(int(angle))+".png"

    #angle_filename = path_image+'\\'+study_uid+'mip'+"."+str(int(angle))+".png" #rajouter le study iud du patient ou numero de serie 
    angle_filename = os.path.join(path_image, filename)
    f.savefig(angle_filename, bbox_inches='tight')
    plt.close()   
    return angle_filename

def mip_imshow_4D(mask, angle, cmap) :
    """function to visualize MIP of 4D image

    Args:
        mask ([type]): [description]
        angle ([type]): [description]
        cmap ([type]): [description]
    """
    liste = []
    number_roi = mask.shape[3]
    for roi in range(number_roi): 
        #print(roi)
        #pour mettre en coronal 
        liste.append(np.transpose(np.flip(mask[:,:,:,roi], axis = 2), (2,1,0)))
    
    new_mask = np.stack((liste), axis = 3)
    vol_angle = scipy.ndimage.interpolation.rotate(new_mask , angle , reshape=False, axes = (2,3))
    MIP = np.amax(vol_angle,axis=3)
    MIP2 = np.amax(MIP,axis=2)

    f = plt.figure(figsize=(10,10))
    axes = plt.gca()
    axes.set_axis_off()

    plt.imshow(MIP2, cmap = cmap )
    



def mip_projection_4D(mask, angle, path_image, study_uid, number_roi, cmap):
    """function to save MIP of 4D array

    Args:
        mask ([ndarray]): [4D matrix]
        angle ([type]): [description]
        path_image ([type]): [description]
        study_uid ([type]): [description]
        number_roi ([type]): [description]
        cmap ([type]): [description]

    Returns:
        save as png the MIP image
    """

    mask_3d = np.amax(mask, axis = 3)



    numpy_array = np.transpose(np.flip(mask_3d, axis = 2), (2,1,0)) #coronal

    vol_angle = scipy.ndimage.interpolation.rotate(numpy_array , angle , reshape=False, axes = (1,2))
    MIP = np.amax(vol_angle,axis=2)

    f = plt.figure(figsize=(10,10))
    axes = plt.gca()
    axes.set_axis_off()

    plt.imshow(MIP, cmap = cmap)
    filename = study_uid+'mip_MASK'+"_"+str(int(angle))+".png"
    #angle_filename = path_image+'\\'+study_uid+'mip_MASK'+"."+str(int(angle))+".png" #rajouter le study iud du patient ou numero de serie 
    angle_filename = os.path.join(path_image, filename)
    f.savefig(angle_filename, bbox_inches='tight')
    plt.close()        
    return angle_filename
        

def create_pdf_mip(angle_filenames, output_path_name) : 
    """angle_filenames : [[path_mip_PET 1, path_mip_MASK 1, study_uid], [path_mip_PET 2, path_mip_MASK 2, study_uid],... ]

    """

    pdf = FPDF()
    for mip in angle_filenames : 
        pdf.add_page()
        pdf.image(mip[0], x = 0, y = 10, w = 100, h = 190)
        pdf.image(mip[1], x = 100, y = 10, w = 100, h = 190)
        pdf.set_font("Arial", size=12)
    
        pdf.cell(200, 0, txt= str(mip[2]), ln=2, align="C")

    pdf.output(output_path_name)

    return None 

   

def create_mip_gif(numpy_array, path_image, study_uid, type, cmap, borne_max):
    """return a gif of a numpy_array MIP 

    """
    duration = 0.1
    number_images = 60
    
    angle_filenames = []
    
    angles = np.linspace(0, 360, number_images)
    for angle in angles:
        path = mip_projection(numpy_array, angle, path_image, study_uid, type, cmap, borne_max = borne_max)
        angle_filenames.append(path)
              

    create_gif(angle_filenames, duration, path_image, name)

    for image in angle_filenames : 
        os.remove(image)
       
    
    return None 


def mip_superposition_show(mask_array, pet_array, angle, study_uid, vmin, vmax, cmap_pet, cmap_mask, alpha, save = False, directory = None):
    """Show and save as png PET/MASK MIP for one angle

    Args:
        mask_array ([ndarray]): [Mask ndarray 3D]
        pet_array ([ndarray]): [PET ndarray]
        angle ([int]): [angle]
        vmin ([int]): [minimum value]
        vmax ([int]): [maximum value]
        cmap_pet ([str]): [Mask pet]
        cmap_mask ([str]): [Mask cmap]
        save (bool, optional): [True if save the figure]. Defaults to False.
        directory ([str], optional): [Path of the directory]. Defaults to None.

    Returns:
        Save MIP in directory
    """
    mask_array = np.transpose(np.flip(mask_array, axis = 2), (2,1,0)) #coronal
    vol_angle_mask = scipy.ndimage.interpolation.rotate(mask_array , angle , reshape=False, axes = (1,2))
    MIP_mask = np.amax(vol_angle_mask,axis=2)

    pet_array = np.transpose(np.flip(pet_array, axis = 2), (2,1,0)) #coronal
    vol_angle_pet = scipy.ndimage.interpolation.rotate(pet_array , angle , reshape=False, axes = (1,2))
    MIP_pet = np.amax(vol_angle_pet,axis=2)

    f = plt.figure(figsize=(10,10))
    axes = plt.gca()
    axes.set_axis_off()
    plt.imshow(MIP_pet, vmin=vmin, vmax=vmax, cmap=cmap_pet)
    plt.imshow(np.where(MIP_mask, 0, np.nan), cmap=cmap_mask, alpha = alpha)

    if save == True : 
        filename = study_uid + '_PET_MASK'+"_"+str(int(angle))+".png"
        angle_filename = os.path.join(directory, filename)
        f.savefig(angle_filename, bbox_inches='tight')
        plt.close()
        return angle_filename



def mip_superposition_gif(mask_array, pet_array, vmin, vmax, cmap_pet, cmap_mask, directory, name):
    """Generate PET/MASK MIP gif 

    Args:
        mask_array ([ndarray]): [description]
        pet_array ([ndarray]): [description]
        vmin ([int]): [description]
        vmax ([int]): [description]
        cmap_pet ([str]): [description]
        cmap_mask ([str]): [description]
        directory ([str]): [description]
        name ([str]): [description]
    """
    duration = 0.1
    number_images = 60

    angles = np.linspace(0, 360, number_images)
    angles_filenames = []
    for angle in angles:
        angles_filenames.append(mip_superposition_show(mask_array, pet_array, angle, vmin, vmax, cmap_pet, cmap_mask, save = True, directory = directory))
        
    create_gif(angles_filenames, duration, name, directory)

    for image in angles_filenames : 
        os.remove(image)






from library_dicom.dicom_processor.model.Series import Series
from library_dicom.dicom_processor.model.reader.Instance import Instance 
import os 
from math import exp, log, pow
from datetime import datetime, timedelta

class SeriesPT(Series):
    """Specific methods to read PET series
    Store radiopharmaceutical data
    Check Attenuation conrrection
    SUV / SUL calculation
    Get Series as 32bits Nifti SUV/SUL
    

    Arguments:
        Series {[class]} -- [description]
    """

    def __init__(self, path, sul_value=False):
        """Constructor

        Arguments:
            path {String} -- Path folder of series

        Keyword Arguments:
            sul_value {bool} -- [If true return NIFTI SUL] (default: {False})
        """
        super().__init__(path)
        self.sul_value=sul_value




    def get_minimum_acquisition_datetime(self):
        """Get earlier acquisition datetime of the PET serie

        Returns : 
            [datetime] -- [Return the earlier acquisition datetime, or "Undefined" if no acquisition_time or no acquisition date]
        """
        liste = []
        try : 
            for filename in self.file_names : 
                instanceData = Instance(os.path.join(self.path,filename), load_image=True)
                datetime = instanceData.get_acquisition_date() + instanceData.get_acquisition_time()
                parsed_datetime = self.__parse_datetime(datetime)
                liste.append(parsed_datetime)

            return min(liste)

        except Exception : 
            return "Undefined"
        


    def get_series_details(self):
        """Add Pharmaceuticals data to common series details

        Returns:
            [dict] -- [Return the details of a SeriePT from the first Dicom]
        """
        details = super().get_series_details()
        dicomInstance = self.get_first_instance_metadata()
        self.radiopharmaceutical_details = {}
        self.radiopharmaceutical_details = dicomInstance.get_radiopharmaceuticals_tags()
        details['radiopharmaceutical'] = self.radiopharmaceutical_details
        self.pet_correction_details = {}
        self.pet_correction_details = dicomInstance.get_pet_correction_tags()
        details['pet_correction'] = self.pet_correction_details
        self.philips_tag = {}
        self.philips_tag = dicomInstance.get_philips_private_tags()
        details['philips_tags'] = self.philips_tag

        return details
        

    @classmethod
    def __parse_datetime(cls, date_time):
        """ class method to parse datetime

        Returns : 
        [datetime] -- [Return parse datetime]
        """
        #remove microsecond at it is inconstant over dicom
        if '.' in date_time : 
            date_time = date_time[0 : date_time.index('.')]
        #parse datetime to date objet
        return datetime.strptime(date_time, "%Y%m%d%H%M%S")



    def __calculateSUVFactor(self):
        """
        Calcul of  SUV factor

        Returns:
            [float] -- [return SUV factor or "Calcul SUV impossible" if there is "Undefined" value in tags]
        """
        series_details = self.get_series_details()
        units = series_details['series']['Units']
    
        known_units = ['GML', 'BQML', 'CNTS']
        if units not in known_units : 
            raise Exception ('Unknown PET Units')

        if units == 'GML' : return 1
        elif units == 'CNTS' :
            philips_suv_bqml = series_details['philips_tags']['PhilipsBqMlFactor']
            philips_suv_factor = series_details['philips_tags']['PhilipsSUVFactor']
            if (philips_suv_factor != 'Undefined') : return philips_suv_factor
            if (philips_suv_factor == 'Undefined' and philips_suv_bqml == 'Undefined') : raise Exception('Missing Philips private Factors')
        
        patient_weight = series_details['study']['PatientWeight'] * 1000 #kg to g conversion
        
        series_time = series_details['series']['SeriesTime']
        series_date = series_details['series']['SeriesDate']
        series_datetime = series_date + series_time 

        series_datetime = self.__parse_datetime(series_datetime)
        acquisition_datetime = self.get_minimum_acquisition_datetime()
        acquisition_date = series_details['series']['AcquisitionDate']
        
        decay_correction = series_details['series']['DecayCorrection']
        radionuclide_half_life = series_details['radiopharmaceutical']['RadionuclideHalfLife']
        total_dose = series_details['radiopharmaceutical']['TotalDose']

        radiopharmaceutical_start_date_time = series_details['radiopharmaceutical']['RadiopharmaceuticalStartDateTime']

        
        if radiopharmaceutical_start_date_time == 'Undefined' or radiopharmaceutical_start_date_time == '' : 
            #If startDateTime not available use the deprecated statTime assuming the injection is same day than acquisition date
            radiopharmaceutical_start_time = series_details['radiopharmaceutical']['RadiopharmaceuticalStartTime']
            radiopharmaceutical_start_date_time = acquisition_date + radiopharmaceutical_start_time 
            
        
        radiopharmaceutical_start_date_time = self.__parse_datetime(radiopharmaceutical_start_date_time)
        
        if (total_dose == 'Undefined' or total_dose == None or  acquisition_datetime== 'Undefined' 
            or patient_weight == 'Undefined' or patient_weight == 'None' or radionuclide_half_life == 'Undefined' ) :
            raise Exception('Missing Radiopharmaceutical data or patient weight')
        
        #Determine Time reference of image acqusition 
        acquisition_hour = series_datetime
        if (acquisition_datetime != 'Undefined'
             and (acquisition_datetime - series_datetime).total_seconds() < 0 and units == 'BQML') : 
            acquisition_hour = acquisition_datetime

        
        #Calculate decay correction
        if decay_correction == 'START' : 
            delta = (acquisition_hour - radiopharmaceutical_start_date_time)
            delta = delta.total_seconds()
            if (delta < 0) : raise("Acqusition time before injection time")
            decay_factor = exp(-delta * log(2) / radionuclide_half_life)

        #If decay corrected from administration time no decay correction to apply
        elif decay_correction == 'ADMIN' : 
            decay_factor = 1


        else : raise Exception('Unknown Decay Correction methode')
        
        suv_conversion_factor = 1/((total_dose * decay_factor) / patient_weight)
        
        if units == 'CNTS' : return philips_suv_bqml * suv_conversion_factor
        else : return suv_conversion_factor


    def calculateSULFactor(self):
        """Calcul SUL Factor

        Returns:
            [float] -- [Return SUL Factor or "Calcul SUL impossible" if there is "Undefined" value in tags]
        """
        series_details = self.get_series_details()
        patient_sex = series_details['patient']['PatientSex']
        patient_weight = series_details['study']['PatientWeight']
        patient_height = series_details['study']['PatientHeight']
        if (patient_sex == 'Undefined' or patient_height == 'Undefined' or patient_weight == 'Undefined' or patient_height == 0.0) : 
            raise Exception('Missing Height or Weight to calculate SUL')
        bmi =  patient_weight / pow(patient_height, 2)
        if patient_sex == 'F' :
            return 9270 / (8780 + 244 * bmi)
        elif patient_sex == 'M':
            return 9270 / (6680 + 216 * bmi)
        else :
            raise Exception('Unknown Sex String')
    

    def is_corrected_attenuation(self):
        """If PET Series is attenuation corrected

        Returns:
            [bool] -- [True if attenuation corrected]
        """
        series_details = self.get_series_details()
        corrected_image = series_details['pet_correction']
        if 'ATTN' in corrected_image : return True
        else : return False


    
    def get_numpy_array(self):
        """[summary]

        Returns:
            [array] -- [return array of the SeriesPT with SUV and SUL factor in 32bis npArray ]
        """
        numpy_array = super().get_numpy_array()
        try:
            if (self.sul_value == False) :
                return numpy_array * self.__calculateSUVFactor()
            else :
                return numpy_array * self.__calculateSUVFactor() * self.calculateSULFactor()
        except Exception as err:
            print("Error generating result array", err)